// Standoff DLL - By Mario Brito (HCl) - mbrito@student.dei.uc.pt
// 11-April-2004

#include "stdafx.h"
#include "wcsohr.h"
#include "decore.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifndef PI
#define PI 3.14159265358979323
#endif

// small structure for loaded TREs (created by fans), and their files... may come in handy ;)
#define MAX_TRES 200

typedef struct
{
	unsigned long CRC;
	int size;
	int offset;
	int csize;
	char name[256];
} TRE_ENTRY;

typedef struct
{
	char name[200];
	int nFiles;
	TRE_ENTRY * file;
} TRE;
int num_tres = 0;

TRE tre_list[MAX_TRES];


// Standoff's (slightly improved) VDU patch
char * vdu_switch_index = (char *) 0x4989dc;
int * vdu_switch_addr = (int *) 0x498964;
int * vdu_switch_newaddr = (int *) 0x498a14;

void * vdu_continue = (void *) 0x497f91;

typedef struct
{
	int number;
	int mat;
} VDU_ENTRY;

VDU_ENTRY vdu_list[100];
int vdu_list_len = 0;

int __stdcall get_mat_from_vdu(int vdu)
{
	for (int i = 0; i < vdu_list_len; i ++)
		if (vdu == vdu_list[i].number)
			return vdu_list[i].mat;

	__asm int 3;	// should never happen!!!
}

int _vdu;
void __declspec(naked) vdu_intercept(void)
{
	__asm
	{
		//int 3

		pushad
		push eax
		call get_mat_from_vdu
		mov _vdu, eax
		popad
		push _vdu
		jmp vdu_continue
	}
}

void vdu_install(void)
{
	FILE * f = NULL;
	char * fileName = "config\\vdu.cfg";
	char s1[100];
	char s2[100];

	
	*vdu_switch_newaddr = (int)vdu_intercept;
	
	for (int nTre = 0; nTre < num_tres; nTre ++)
		for (int nFile = 0; nFile < tre_list[nTre].nFiles; nFile ++)
			if ( !_strcmpi (tre_list[nTre].file[nFile].name, fileName) )
			{
				f = fopen (tre_list[nTre].name, "rb");
				fseek(f, tre_list[nTre].file[nFile].offset, SEEK_SET);
				goto file_open;
			}

	// if not found, try to open folder file
	f = fopen (fileName, "rb");
	if (f == NULL) return;

	file_open:

	while (1)
	{
		fscanf (f, "%s %s\n", s1, s2);
		if (!strnicmp(s1, "end",3)) break;
		else 
		{
			// store new entry on list
			vdu_list[vdu_list_len].number = atoi(s1);
			vdu_list[vdu_list_len].mat = atoi(s2);

			// patch vdu switch table
			vdu_switch_index[ vdu_list[vdu_list_len].number - 1 ] = 44;

			vdu_list_len ++;

			if (vdu_list_len == 100) __asm int 3
		}
	}

	fclose (f);
}


// gun convergence patch
int converge_gun;
int g_index;
int g_x;
int n_guns;
typedef struct
{
	float x,y,z;
} VECTOR;

typedef struct
{
	VECTOR v1;
	int x;
	VECTOR v2;
	int y;
	VECTOR v3;
	int z;
} MATRIX34;

typedef struct
{
	VECTOR v1,v2,v3;
} MATRIX33;


VECTOR v[5];
VECTOR * o;
int vector_number = 0;

void * scatter_return = (void *) 0x4403a0;
void * scatter_skip_return = (void *) 0x43fffb;
void * scatter_skip_return2 = (void *) 0x4401ab;
void * scatter_skip_call = (void *)0x40dbd0;
void * scatter_rotate_return = (void *)0x40d51f;

typedef struct
{
	int bullet;
	float angle;
} CG_ENTRY;		// convergence gun entry

CG_ENTRY cg[100];
int cg_len = 0;

//#define SCATTER_INDEX 3  // Standoff's massdriver (test)

//#define SCATTER_ANGLE ((2.0f*PI)/180.0f)
float SCATTER_ANGLE = 0.0f;

int __stdcall cg_setup (int bullet)
{
	for (int i = 0; i < cg_len; i ++)
		if (bullet == cg[i].bullet)
		{
			SCATTER_ANGLE = (cg[i].angle*PI)/180.0f;
			return 1;
		}

	return 0;
}

// This function actually makes the bolts rotate ( "scatter" )
void rot(MATRIX34 * src, MATRIX34 * dest, int g)
{
	MATRIX33 m;

	memcpy (&(m.v1), &(src->v1), sizeof(VECTOR));	// copy vectors to temp location
	memcpy (&(m.v2), &(src->v2), sizeof(VECTOR));
	memcpy (&(m.v3), &(src->v3), sizeof(VECTOR));

	if (g > 0)
	{
		dest->v1.x = m.v1.x * cos(SCATTER_ANGLE) + m.v1.z * sin(SCATTER_ANGLE);
		dest->v1.y = m.v1.y;
		dest->v1.z = - m.v1.x * sin(SCATTER_ANGLE) + m.v1.z * cos(SCATTER_ANGLE);

		dest->v2.x = m.v2.x * cos(SCATTER_ANGLE) + m.v2.z * sin(SCATTER_ANGLE);
		dest->v2.y = m.v2.y;
		dest->v2.z = - m.v2.x * sin(SCATTER_ANGLE) + m.v2.z * cos(SCATTER_ANGLE);

		dest->v3.x = m.v3.x * cos(SCATTER_ANGLE) + m.v3.z * sin(SCATTER_ANGLE);
		dest->v3.y = m.v3.y;
		dest->v3.z = - m.v3.x * sin(SCATTER_ANGLE) + m.v3.z * cos(SCATTER_ANGLE);
	}
	else
	{
		dest->v1.x = m.v1.x * cos(-SCATTER_ANGLE) + m.v1.z * sin(-SCATTER_ANGLE);
		dest->v1.y = m.v1.y;
		dest->v1.z = - m.v1.x * sin(-SCATTER_ANGLE) + m.v1.z * cos(-SCATTER_ANGLE);

		dest->v2.x = m.v2.x * cos(-SCATTER_ANGLE) + m.v2.z * sin(-SCATTER_ANGLE);
		dest->v2.y = m.v2.y;
		dest->v2.z = - m.v2.x * sin(-SCATTER_ANGLE) + m.v2.z * cos(-SCATTER_ANGLE);

		dest->v3.x = m.v3.x * cos(-SCATTER_ANGLE) + m.v3.z * sin(-SCATTER_ANGLE);
		dest->v3.y = m.v3.y;
		dest->v3.z = - m.v3.x * sin(-SCATTER_ANGLE) + m.v3.z * cos(-SCATTER_ANGLE);
	}

	dest->x = src->x;  // copy the start position
	dest->y = src->y;
	dest->z = src->z;
}


// This function handles the bolts rotation 
void __declspec (naked) scatter_rotate()
{

	__asm cmp converge_gun, 0
	__asm jz dont_rotate

	__asm lea esi, dword ptr[eax+0x30]

	__asm pushad
	
	__asm push g_x 
	__asm push edi
	__asm push esi
	__asm call rot
	__asm add esp, 12

	__asm mov converge_gun, 0

	__asm popad


	__asm jmp scatter_rotate_return


dont_rotate:
	__asm lea esi, dword ptr[eax+0x30]
	__asm repz movsd
	__asm jmp scatter_rotate_return
}

// gun convergence is loosely based on the ScatterGun patch, but some functions aren't used
void __declspec (naked) scatter_gun() 
{
	__asm pushad

	__asm mov converge_gun, 0
	__asm push dword ptr [esi]
	__asm call cg_setup
	__asm cmp eax, 1
	__asm jnz leave_scatter

	__asm mov converge_gun, 1

	__asm mov o, esi
	__asm add o, 4
		
	__asm mov eax, o
	__asm mov eax, [eax]
	__asm mov n_guns, eax
	__asm add o, 4

	__asm mov eax, dword ptr[esi+0x50]
	__asm add eax, dword ptr [ebp-0xc]
	__asm mov edx, 0
	__asm idiv n_guns
	__asm mov g_index, edx
	
	__asm mov eax, 12
	__asm mov edx, 0
	__asm mul g_index
	__asm add eax, o
	__asm mov eax, [eax]
	__asm mov g_x, eax

leave_scatter:
	__asm popad
	__asm mov eax, dword ptr [ebp-0x14]
	__asm test eax, eax
	__asm jmp scatter_return

}


void * scatter_return_full = (void *)0x4400d3;
void __declspec (naked) scatter_gun_full()
{
	__asm pushad

	__asm mov converge_gun, 0
	__asm push dword ptr [ebx]
	__asm call cg_setup
	__asm cmp eax, 1
	__asm jnz leave_scatter_full

	__asm mov converge_gun, 1

	__asm mov o, ebx
	__asm add o, 4
		
	__asm mov eax, o
	__asm mov eax, [eax]
	__asm mov n_guns, eax
	__asm add o, 4

	__asm mov eax, dword ptr [ebp-4]
	__asm mov g_index, eax
	
	__asm mov eax, 12
	__asm mov edx, 0
	__asm mul g_index
	__asm add eax, o
	__asm mov eax, [eax]
	__asm mov g_x, eax

leave_scatter_full:
	__asm popad
	__asm mov byte ptr [edi+0x2d4], 1
	__asm jmp scatter_return_full

}

void install_gun_convergence_patch()
{

	char * fileName = "config\\gun_conv.cfg";
	FILE * f = NULL;
	char s1[100];
	char s2[100];

	for (int nTre = 0; nTre < num_tres; nTre ++)
		for (int nFile = 0; nFile < tre_list[nTre].nFiles; nFile ++)
			if ( !_strcmpi (tre_list[nTre].file[nFile].name, fileName) )
			{
				f = fopen (tre_list[nTre].name, "rb");
				fseek(f, tre_list[nTre].file[nFile].offset, SEEK_SET);
				goto file_open;
			}

	// if not found, try to open folder file
	f = fopen (fileName, "rb");
	if (f == NULL) return;

	file_open:

	while (1)
	{
		fscanf (f, "%s %s\n", s1, s2);
		if (!strnicmp(s1, "end",3)) break;
		else 
		{
			// store new entry on list
			cg[cg_len].bullet = atoi(s1);
			cg[cg_len].angle = atof(s2);

			cg_len ++;
			if (cg_len == 100) __asm int 3
		}
	}


	// install the Scatter Gun patch
	// - 5 bolts function
	void (*temp)(void) = (void(*)(void)) ((unsigned int)scatter_gun - (unsigned int)0x4403a0);
	char scatter_bridge[] = {0xe9,0,0,0,0};
	memcpy (scatter_bridge+1, &temp, 4);
	memcpy ((void*)0x44039b, scatter_bridge, 5);

	//  - bolt rotation function
	temp = (void(*)(void)) ((unsigned int)scatter_rotate - (unsigned int)0x40d51f);
	memcpy (scatter_bridge+1, &temp, 4);
	memcpy ((void*)0x40d51a, scatter_bridge, 5);

	// - full guns
	temp = (void(*)(void)) ((unsigned int)scatter_gun_full - (unsigned int)0x4400d1);
	char scatter_bridge2[] = {0xe9,0,0,0,0};
	memcpy (scatter_bridge2+1, &temp, 4);
	memcpy ((void*)0x4400cc, scatter_bridge2, 5);
}



// multiple UV2 patch
void (*uv2_default) () = (void(*)())0x4431e3;
void (*uv2_new)() = (void(*)())0x443557;
void (*uv2_newj)() = (void(*)())0x443762;
char * uv2_string = "custom00";

void __declspec (naked) uv2_hook()
{
	__asm {

//		int 3

		cmp ecx, 8
		ja hook
		jmp uv2_default

hook:
		pushad	// lazy build string
		
		mov edi, uv2_string
		add edi, 6

		mov eax, ecx
		mov edx, 0

		mov ecx, 10		// divide by 10
		div ecx

		add eax, '0'	// convert to ascii
		add edx, '0'
		stosb			// store byte (stosb increments edi too)
		mov eax, edx
		stosb

		popad


		mov ecx, 0x4fe544
		cmp dword ptr [ecx], edi
		jnz newjnz
		push edi
		push uv2_string
		jmp uv2_new
newjnz:
		jmp uv2_newj
		
	}

}


// UV2 hook part 2
void (*uv2e_switch)() = (void(*)())0x44396C;
void (*uv2e_cont_m8)() = (void(*)())0x442C36;
void (*uv2e_addr)() = 0;

void __declspec (naked) uv2_hook2()
{
	__asm 
	{
		pushad
		cmp ecx,8
		ja uv2e_patch
		mov eax, uv2e_switch
		shl ecx,2
		add eax, ecx
		mov eax, [eax]
		mov uv2e_addr, eax
		popad
		jmp dword ptr [uv2e_addr]
		
uv2e_patch:
		popad
		jmp uv2e_cont_m8
	}
}


// UV2 hook part 3 (EXPERIMENTAL!!)
void (*uv3e_switch)() = (void(*)())0x477E38;
void (*uv3e_cont_m8)() = (void(*)())0x477DCF;

void __declspec (naked) uv2_hook3()	// comm sent by player?
{
	__asm 
	{
		cmp ecx,8
		ja uv3e_patch
		xor edx,edx
		mov dl, [0x477E48+ecx]
		jmp dword ptr [uv3e_switch + edx*4]


uv3e_patch:
		jmp uv3e_cont_m8
	}
}


// divx patch (integrated version)
// -----------------------------------------

DEC_PARAM dec_param;
DEC_FRAME dec_frame;
DEC_MEM_REQS decMemReqs;
DEC_SET dec_set;

#define SIZE_OF_DIVX_BUFFERS 900000

char buffer1[SIZE_OF_DIVX_BUFFERS];
char buffer2[SIZE_OF_DIVX_BUFFERS];
char buffer3[SIZE_OF_DIVX_BUFFERS];
char buffer4[SIZE_OF_DIVX_BUFFERS];
char buffer5[SIZE_OF_DIVX_BUFFERS];
char buffer6[SIZE_OF_DIVX_BUFFERS];

//char frame1[300000];
//char frame2[300000];
//char divx_frame[200000];

char frame1[SIZE_OF_DIVX_BUFFERS];
char frame2[SIZE_OF_DIVX_BUFFERS];
char divx_frame[SIZE_OF_DIVX_BUFFERS];

int frame_counter, requested_frame;
FILE * divx_fid;
char fileName[80];

HMODULE h;
int (STDCALL * d)(unsigned long, unsigned long, void *, void *);

typedef struct
{
unsigned long red;
unsigned long green;
unsigned long blue;
} color24l;

typedef struct
{
unsigned char red;
unsigned char green;
unsigned char blue;
} color24;


color24 buffer24[640*480];							// 24 bit buffer


void convert_555_565(short * dest, short * src, int dx, int dy)
{
for (int i = 0; i < dx*dy; i++)
	{
	dest[i] = (src[i] & 0x1f) | 
		      (((src[i] >> 5) & 0x1f)<< 6) | 
			  (((src[i] >> 10) & 0x1f)<<11);
	}
}



void convert_24_16(int dx, int dy, bool convert)
{
int row, col;

memset (frame1, 0, dx*16*sizeof(short));

if (convert)
	for (row = dy%16; row < dy; row++) // fix: remove excess rows in case movie was not multiple of 16 rows
		for (col = 0; col < dx; col++)
			{
			((short *)frame1)[row*dx + col] = (buffer24[row*dx + col].red >> 3) | ((buffer24[row*dx + col].green >> 2)<<5) | ((buffer24[row*dx + col].blue >> 3)<<11);
			}
else
	for (row = dy%16; row < dy; row++) // fix: remove excess rows in case movie was not multiple of 16 rows
		for (col = 0; col < dx; col++)
			{
			((short *)frame1)[row*dx + col] = (buffer24[row*dx + col].red >> 3) | ((buffer24[row*dx + col].green >> 3)<<5) | ((buffer24[row*dx + col].blue >> 3)<<10);
			}
}


void flipi (char *dest, char *src, int dx, int dy)
{
dest += dx * dy * 4;
for (int y = 0; y < dy; y ++)
	{
	dest -= 4*dx;
	src += dx*2;
	for (int x = 0; x < dx; x ++)
		{
		short s;
		s = *((short*)(src+2*x));
		*((short*)(dest+4*x)) = s;
		*((short*)(dest+4*x+2)) = s;
		}
	}
}


void flip (char *dest, char *src, int dx, int dy)
{
dest += dx * dy * 2;
for (int y = 0; y < dy; y ++)
	{
	dest -= 2*dx;
	memcpy (dest, src + dx*y*2, dx*2);
	}
}

int first_frame_start = 0;

void * WVE_continue = (void *)0x4b50f7;
void WVE_Uncompress(void * p1, void * p2)
{
	
	__asm
	{
		pushad

		push p2				// simulate the call
		push p1
		
		push offset return_address

		push ebp
		mov ebp,esp
		push ebx
		mov ebx,[ebp+0xc]
		jmp WVE_continue

		return_address:
		add esp,8
	
		popad
	}
	
}

void movie_player (void * compr, void * s)
{
bool divx;
bool interlaced = false;
bool convert = false;

char * src = ((char *)s);
char * dest = (char *)(*((int *)compr+3));

char * t;

int dx=0;
int dy=0;

// --------------------------------------------------------------
// If the pixel format is 565 instead of 555, the image will look messed up.
// Thanks to KillerWave for noticing this.
//
// I *think* there's no more modes for 16-bit color, other than 555 and 565...
if ( ((char *)compr)[0x30] == 6)
	convert = true;
// --------------------------------------------------------------

memcpy (&dx, src, 2);
memcpy (&dy, src+2, 2);

if (dx != 96)
	interlaced = true;

if (src[8] == 'D' && src[9] == 'I' && src[10] == 'V' && src[11] == 'X')
	{
	divx = true;
	src += 12;
	}
else
	{
	divx = false;
	src += 8;
	}

if (!dest)
	{
	memcpy (((char*)compr)+0x4, &dy, 4);
	memcpy (((char*)compr)+0x8, &dx, 4);
	t = frame1;		memcpy (((char*)compr)+0xc, &t, 4);
	t = frame2;		memcpy (((char*)compr)+0x10, &t, 4);

	if (divx)
		{
		if (divx_fid)
			fclose(divx_fid);

		strcpy (fileName, "divx\\");
		strcat (fileName, src+4);

		// ----------------------------------------------
		// first try: open file in directory
		divx_fid = fopen (fileName, "rb");
		frame_counter = 0;
		first_frame_start = 0;


		// second try: open file inside tre archive
/*		if (!divx_fid)
			for (int j = 0; j < n_tre_files; j++)
				for (int i = 0; i < tre[j].n_tre_entries; i ++)
					if (!_stricmp(tre[j].tre[i].name, fileName))
					{
						divx_fid = fopen (tre[j].filename, "rb");
						fseek (divx_fid, tre[j].tre[i].offset, SEEK_SET);
						first_frame_start = tre[j].tre[i].offset;
						goto file_open;
					}
*/

			// see if file exists in any of the loaded TREs
		for (int nTre = 0; nTre < num_tres; nTre ++)
			for (int nFile = 0; nFile < tre_list[nTre].nFiles; nFile ++)
				if ( !_strcmpi (tre_list[nTre].file[nFile].name, fileName) )
				{
					divx_fid = fopen (tre_list[nTre].name, "rb");
					fseek(divx_fid, tre_list[nTre].file[nFile].offset, SEEK_SET);
					first_frame_start = tre_list[nTre].file[nFile].offset;
					goto file_open;
				}




file_open:

		// ----------------------------------------------



		dec_param.x_dim = dx;
		dec_param.y_dim = dy;
		dec_param.output_format = DEC_RGB24;
		dec_param.time_incr = 15; // 15 fps?

		//decore(1, DEC_OPT_MEMORY_REQS, &dec_param, &decMemReqs);
		d(1, DEC_OPT_MEMORY_REQS, &dec_param, &decMemReqs);

		if (decMemReqs.mp4_display_buffers_size > SIZE_OF_DIVX_BUFFERS)
			{
			FILE * f = fopen ("c:\\windows\\desktop\\limits.txt", "at");
			fprintf (f, "Buffer overflow, decMemReqs.mp4_display_buffers_size = %d\n",decMemReqs.mp4_display_buffers_size);
			fclose(f);
			}

		if (decMemReqs.mp4_edged_for_buffers_size > SIZE_OF_DIVX_BUFFERS)
			{
			FILE * f = fopen ("c:\\windows\\desktop\\limits.txt", "at");
			fprintf (f, "Buffer overflow, decMemReqs.mp4_edged_for_buffers_size = %d\n",decMemReqs.mp4_edged_for_buffers_size);
			fclose(f);
			}

		if (decMemReqs.mp4_edged_ref_buffers_size > SIZE_OF_DIVX_BUFFERS)
			{
			FILE * f = fopen ("c:\\windows\\desktop\\limits.txt", "at");
			fprintf (f, "Buffer overflow, decMemReqs.mp4_edged_ref_buffers_size = %d\n",decMemReqs.mp4_edged_ref_buffers_size);
			fclose(f);
			}

		if (decMemReqs.mp4_state_size > SIZE_OF_DIVX_BUFFERS)
			{
			FILE * f = fopen ("c:\\windows\\desktop\\limits.txt", "at");
			fprintf (f, "Buffer overflow, decMemReqs.mp4_state_size = %d\n",decMemReqs.mp4_state_size);
			fclose(f);
			}

		if (decMemReqs.mp4_stream_size > SIZE_OF_DIVX_BUFFERS)
			{
			FILE * f = fopen ("c:\\windows\\desktop\\limits.txt", "at");
			fprintf (f, "Buffer overflow, decMemReqs.mp4_stream_size = %d\n",decMemReqs.mp4_stream_size);
			fclose(f);
			}

		if (decMemReqs.mp4_tables_size > SIZE_OF_DIVX_BUFFERS)
			{
			FILE * f = fopen ("c:\\windows\\desktop\\limits.txt", "at");
			fprintf (f, "Buffer overflow, decMemReqs.mp4_tables_size = %d\n",decMemReqs.mp4_tables_size);
			fclose(f);
			}

		dec_param.buffers.mp4_edged_ref_buffers = buffer1;
		dec_param.buffers.mp4_edged_for_buffers = buffer2;
		dec_param.buffers.mp4_display_buffers = buffer3;
		dec_param.buffers.mp4_state = buffer4;
		dec_param.buffers.mp4_tables = buffer5;
		dec_param.buffers.mp4_stream = buffer6;

		memset(buffer1, 0, SIZE_OF_DIVX_BUFFERS);
		memset(buffer2, 0, SIZE_OF_DIVX_BUFFERS);
		memset(buffer3, 0, SIZE_OF_DIVX_BUFFERS);
		memset(buffer4, 0, SIZE_OF_DIVX_BUFFERS);
		memset(buffer5, 0, SIZE_OF_DIVX_BUFFERS);
		memset(buffer6, 0, SIZE_OF_DIVX_BUFFERS);

		//decore(1, DEC_OPT_INIT, &dec_param, NULL);
		d(1, DEC_OPT_INIT, &dec_param, NULL);

		//decore(1, DEC_OPT_SETOUT, &dec_param,NULL);
		d(1, DEC_OPT_SETOUT, &dec_param,NULL);

		dec_set.postproc_level = 99;
		//decore(1, DEC_OPT_SETPP, &dec_set, NULL);
		d(1, DEC_OPT_SETPP, &dec_set, NULL);
		}
	}

if (divx && divx_fid)
	{
	memcpy(&requested_frame, src, 4);

	// ATTENTION: WCP may skip frames at it's discretion. However, DivX CANNOT EVER skip
	// frames, so we gotta uncompress them all :(

	// Added support for looping comms
	decode:
	while (frame_counter <= requested_frame && divx)
		{
		int size, l;
		fread (&size, 4, 1, divx_fid);
		l = fread (divx_frame, size, 1, divx_fid);
		if (l == 0)
			{
			//fclose(divx_fid); // removed due to looping code
			//divx_fid = 0;
			break;
			}

		dec_frame.bitstream = divx_frame;
		dec_frame.bmp = buffer24;
		dec_frame.length = size;	
		dec_frame.render_flag = 1;	
		dec_frame.stride = dx;		
		//decore(1, 0, &dec_frame, 0);
		d(1, 0, &dec_frame, 0);
		
		frame_counter ++;
		}
	
	if (requested_frame < frame_counter - 1) // looping
	{
		frame_counter = 0;
		fseek (divx_fid, first_frame_start, SEEK_SET);
		goto decode;	// ugly goto...
	}

	convert_24_16(dx, dy, convert);

	if (interlaced)
		flipi(frame2, frame1, dx, dy);
	else
		flip (frame2, frame1, dx, dy);
	}
else if (!divx)
	{
	// Not DivX, fall back to Origin code
	WVE_Uncompress(compr, s);
	}
}


void install_divx_patch()
{
	char * mem = (char *)0x4b50f0;		//address on Gold EXE 
	char * f = (char *) movie_player;
	char bridge[15] = {0x90,0xe9,0,0,0,0,0xc3};
	memcpy (mem, bridge, 7);
	int disp = ((int)movie_player) - 0x4b50f6;
	memcpy (mem+2, &disp, 4);

	h = LoadLibrary("divx.dll");
	d = (int (STDCALL*)(unsigned long, unsigned long, void *, void *))GetProcAddress(h, "decore");
}
// (end of divx patch)----

// keyboard patch
void * get_register_value = (void *) 0x4885c0;
void * kb_mci_return_success = (void *) 0x48736d;
void * kb_mci_return_fail = (void *)	0x4854eb;

int kb_table[256];

void * keyboard_hook_continue = (void *) 0x44637f;

void __declspec (naked) keyboard_hook()
{
	__asm 
	{
		// hook
		pushad

		mov ebx, dword ptr [eax]
		cmp ebx, 4
		jnz not_keypress

		add eax, 0xc
		mov ebx, dword ptr [eax]
		
		inc kb_table[ebx*4]

not_keypress:

		popad

		// continue
		xor ebx, ebx
		mov edx, eax
		add eax, 0x14

		jmp keyboard_hook_continue		
	}
}

void __declspec (naked) kb_mci_hook()
{

	__asm cmp eax, 0xd8ce4d00				// void HCL_KeyboardReset(void)
	__asm jnz fail_1

	__asm pushad
	__asm mov ebx,0
_cycle_:
	__asm mov kb_table[ebx*4], 0
	__asm inc ebx
	__asm cmp ebx, 256
	__asm jne _cycle_
	__asm popad

	__asm jmp success

fail_1:
	__asm cmp eax, 0xd07ed004				// int HCL_KeyboardPoll(int char_value)
	__asm jnz fail

	__asm pushad
	__asm mov eax, dword ptr[edi+4]			// get parameter (int)
	__asm push eax
	__asm mov ecx, ebx
	__asm call get_register_value

	__asm mov eax, kb_table[eax*4]

	__asm mov dword ptr[ebx+0x6c], eax		// return value to interpreter
	__asm popad

	__asm jmp success
	
fail:
	__asm cmp eax, 0x30ebb00				// it wasn't any of the new functions... so, continue usual SF_ functions
	__asm jmp kb_mci_return_fail


success:
	__asm jmp kb_mci_return_success
}



void add_tre_to_list(char * name)
{
	FILE * f = fopen (name, "rb");
	if (f)
	{
		strcpy ( tre_list[num_tres].name, name );
		fread ( &tre_list[num_tres].nFiles, 4, 1, f);
		fseek (f, 16, SEEK_SET);
		tre_list[num_tres].file = (TRE_ENTRY *) malloc (tre_list[num_tres].nFiles * sizeof(TRE_ENTRY));
		fread (tre_list[num_tres].file, tre_list[num_tres].nFiles * sizeof(TRE_ENTRY), 1, f);
		fclose(f);

		num_tres ++;
	}
}


// footer addon (part of fiction patch)
typedef struct
{
	int x,y;
} FOOTER;

FOOTER footer[3];

void __stdcall set_footer_coords(int x, int y, int res)
{
	if (res == 640)
	{
		footer[0].x = x;
		footer[0].y = y;
	}

	else if (res == 800)
	{
		footer[1].x = x;
		footer[1].y = y;
	}

	else if (res == 1024)
	{
		footer[2].x = x;
		footer[2].y = y;
	}


}

// fiction patch
int page_number = -1;

int area_rect[4];

int loaded_page_number = -1;
char *page = NULL;

int fonts_created = 0;
void * fiction_font[10];
int fiction_variable[10];

void * (*wcso_create_font)(char *, int, int, int) = (void*(*)(char *, int, int, int)) 0x449250;
void * (*wcso_get_section)(char *) = (void*(*)(char *)) 0x4491c0;

void * fp_leave_addr = (void *)0x4933cc;
void * fp_continue_room_tag = (void *)0x49334f;

void * wcso_print_char = (void *) 0x438040;
void * wcso_char_len = (void *)0x438be0;

// assume each tab is equivalent to 8 spaces
#define TAB_SPACES 8

void _stdcall set_fiction_area_rect (int minx, int maxx, int miny, int maxy)
{
	int * res_x = (int*)0x43d3c6;
	int * res_y = (int*)0x43d3cf;

	float frect[4];

	frect[0] = minx / 10.0f;
	frect[1] = maxx / 10.0f;
	frect[2] = miny / 10.0f;
	frect[3] = maxy / 10.0f;

	frect[0] /= 100.0f;
	frect[1] /= 100.0f;
	frect[2] /= 100.0f;
	frect[3] /= 100.0f;

	area_rect[0] = frect[0] * *res_x;
	area_rect[1] = frect[1] * *res_y;
	area_rect[2] = frect[2] * *res_x;
	area_rect[3] = frect[3] * *res_y;
}


void __stdcall set_fiction_var(int value, int variable)
{
	if (variable < 10 && variable >= 0)
		fiction_variable[variable] = value;
}


void print_char (char c, int x, int y, void * font)
{
	__asm
	{
		pushad

		mov ecx, font
		push y
		push x
		movzx eax, c
		push eax
		call wcso_print_char

		popad
	}
}

int character_length(char c, void * font)
{
	int val = 0;

	__asm
	{
		pushad

		mov ecx, font
		movzx eax, c
		push eax
		call wcso_char_len
		mov val, eax

		popad
	}

	return val;	
}



int length_word(char * word, void * font)
{
	int len = 0;

	for (int i = 0; word[i] != ' ' && word[i] != '\0'; i ++)
	{
		if (word[i] == '#')
		{
			i++;

			if (word[i] >= '0' && word[i] <= '9')	// color change, irrelevant
				continue;
			if (word[i] == 'c')		// check size of callsign and add
			{

				char * callsign = *(char **)0x4ff89c;
				while (*callsign != '\0')
				{
					len += character_length(*callsign++, font);
				}

				continue;
			}
			if (word[i] == 'v')		// check size of value of var after being converted to text
			{
				i ++;

				char s[80];
				int index = word[i] - '0';

				sprintf (s, "%d", fiction_variable[index]);
				
				for (int j = 0; s[j] != '\0'; j++)
				{
					len += character_length(s[j], font);
				}

				continue;
			}
		}
		len += character_length(word[i], font);
	}

	return len;
}


void print_string(int x, int y, int maxx, int maxy, char * string)
{

	char temp_char = 0;
	void * temp_font = fiction_font[0];
	int xx = x;
	int yy = y;

	for (int i = 0; string[i] != 0; i ++)
	{
		if (string[i] == '#')	// escape char
		{
			i ++;
			
			if (string[i] >= '0' && string[i] <= '9')
			{
				temp_font = fiction_font[string[i] - '0'];
				continue;
			}

			if (string[i] == 'c')		// -- callsign --
			{
				char * callsign = *(char **)0x4ff89c;
				while (*callsign != '\0')
				{
					temp_char = *callsign++;
					print_char(temp_char, xx, yy, temp_font);
					xx += character_length(temp_char, temp_font);
				}

				continue;
			}

			if (string[i] == 'v')		// -- variable --
			{
				i ++;

				char s[80];
				int index = string[i] - '0';

				sprintf (s, "%d", fiction_variable[index]);
				
				for (int j = 0; s[j] != '\0'; j++)
				{
					temp_char = s[j];
					print_char(temp_char, xx, yy, temp_font);
					xx += character_length(temp_char, temp_font);
				}

				continue;
			}

			if (string[i] == 'f')		// -- footer --
			{
				int * res_x = (int*)0x43d3c6;
				int * res_y = (int*)0x43d3cf;

				if (*res_x == 640)
				{
					xx = footer[0].x;
					yy = footer[0].y;
				}
				else if (*res_x == 800)
				{
					xx = footer[1].x;
					yy = footer[1].y;
				}
				else if (*res_x == 1024)
				{
					xx = footer[2].x;
					yy = footer[2].y;
				}

				continue;
			}

		}
		else if (string[i] == '\t')	// tab
		{
			for (int j = 0; j < TAB_SPACES; j ++)
				xx += character_length(' ', temp_font);
			continue;
		}
		else if (string[i] == 0xa || string[i] == 0xd)	// carriage return
		{
			xx = x;
			yy += 10;

			if (yy > maxy) return;

			if (string[i+1] == 0xa || string[i+1] == 0xd)	i ++;
			continue;
		}

		temp_char = string[i];

		if (temp_char == ' ' && length_word(string + i + 1, temp_font) + character_length(' ', temp_font) > maxx - xx)
		{
			xx = x;
			yy += 10;
	
			continue;
		}

		print_char(temp_char, xx, yy, temp_font);
		xx += character_length(temp_char, temp_font);
	}
}


char * load_page(int n)
{
	char * aux;
	char fileName[200];

	sprintf(fileName, "fiction\\%08d.txt", n);

	// see if file exists in any of the loaded TREs
	for (int nTre = 0; nTre < num_tres; nTre ++)
		for (int nFile = 0; nFile < tre_list[nTre].nFiles; nFile ++)
			if ( !_strcmpi (tre_list[nTre].file[nFile].name, fileName) )
			{
				FILE * f = fopen (tre_list[nTre].name, "rb");

				if (!f) continue;	// shouldn't happen, but....

				fseek(f, tre_list[nTre].file[nFile].offset, SEEK_SET);

				aux = (char *)malloc(tre_list[nTre].file[nFile].size + 1);
				fread(aux, tre_list[nTre].file[nFile].size, 1, f);
				aux[tre_list[nTre].file[nFile].size] = 0;

				fclose(f);

				return aux;
			}

	// if not, load from the folder
	FILE * f = fopen(fileName, "rb");
	if (!f) return NULL;

	int size;

	fseek (f, 0, SEEK_END);
	size = ftell(f);
	fseek (f, 0, SEEK_SET);
	
	aux = (char *)malloc(size + 1);
	fread(aux, size, 1, f);
	aux[size] = 0;

	fclose(f);

	return aux;
}


void print_fiction_page()
{
	if (page_number == -1) return;

	// testes
	//print_string(100, 150, "Mais uns testes de texto do fiction. Testando #1Cores#0... e #2funciona#0! :-)\nUm caracter \"##\" seguido de um numero muda a cor\nPor exemplo: ##1Teste##0 -> #1Teste#0");
	//print_string(100, 200, "Outro exemplo: ##1a##2b##1c##0d -> #1a#2b#1c#0d");
	//print_string(100, 220, "A minha callsign  #2#c#0 e a variavel de fiction ##v0 tem o valor #1#v0#0.");

	if (page_number != loaded_page_number)
	{
		if (page)
		{
			free (page);
			page = 0;
		}

		page = load_page(page_number);
		
		if (page)
			loaded_page_number = page_number;
		else
		{
			loaded_page_number = -1;
			return;
		}
	}

	print_string(area_rect[0], area_rect[1], area_rect[2], area_rect[3], page);
}

void create_font(void)
{
	fonts_created = 1;
	fiction_font[0] = wcso_create_font("ansi08aa", 0xf8,0xf8,0xf8);
	fiction_font[1] = wcso_create_font("ansi08aa", 48,248,248);
	fiction_font[2] = wcso_create_font("ansi08aa", 0, 160, 160);
	
	fiction_font[3] = wcso_create_font("ansi08aa", 0, 244, 0);
	fiction_font[4] = wcso_create_font("ansi08aa", 244,244,0);
	fiction_font[5] = wcso_create_font("ansi08aa", 244, 0, 0);
}

void __declspec(naked) fiction_printer_intercept()
{
	__asm
	{
		// check if we need to build our fonts.. It can probably be done earlier (TODO)
		pushad
		cmp fonts_created, 0
		jnz dont_create_font
		call create_font
		dont_create_font:

		// see if we need have fiction text to print
		cmp page_number, -1
		jz skip_fiction_text

		call print_fiction_page		// if so, print it.

		skip_fiction_text:
		popad

		// continue usual operations
		cmp dword ptr [edi+0x54], 0xffffffff
		je fp_leave
		jmp fp_continue_room_tag
		fp_leave:
		jmp fp_leave_addr

	}
}

void * gameflow_continue = (void *)0x4853a7;
void * gameflow_above_offset = (void *)0x485074;
void * gameflow_equal_offset = (void *)0x48504f;
void __declspec(naked) gameflow_function_hook(void)
{
	__asm
	{

		// HCL_SetFictionPage (int) -----------------------------------
		cmp eax, 0x95beba04	
		jnz try2
		mov eax, dword ptr [edi+4]
		push eax
		mov ecx, ebx
		call get_register_value
		mov page_number, eax
		jmp gameflow_continue
		// ------------------------------------------------------------

try2:
		// HCL_SetFictionVar (int, int)--------------------------------
		cmp eax, 0x68a09108
		jnz try3
		mov eax, dword ptr [edi+4]
		push eax
		mov ecx, ebx
		call get_register_value
		push eax
		mov eax, dword ptr [edi+8]
		push eax
		mov ecx, ebx
		call get_register_value
		push eax
		call set_fiction_var
		jmp gameflow_continue
		// ------------------------------------------------------------

try3:
		// HCL_SetFictionAreaPercent (int, int, int, int)---------------------
		cmp eax, 0x2e134810
		jnz try4
		mov eax, dword ptr [edi+16]
		push eax
		mov ecx, ebx
		call get_register_value
		push eax
		mov eax, dword ptr [edi+12]
		push eax
		mov ecx, ebx
		call get_register_value
		push eax
		mov eax, dword ptr [edi+8]
		push eax
		mov ecx, ebx
		call get_register_value
		push eax
		mov eax, dword ptr [edi+4]
		push eax
		mov ecx, ebx
		call get_register_value
		push eax
		call set_fiction_area_rect
		jmp gameflow_continue
		// ------------------------------------------------------------

try4:
		// HCL_SetFooterCoords (int, int, int)---------------------
		cmp eax, 0x6bb6180c
		jnz fail
		mov eax, dword ptr [edi+12]
		push eax
		mov ecx, ebx
		call get_register_value
		push eax
		mov eax, dword ptr [edi+8]
		push eax
		mov ecx, ebx
		call get_register_value
		push eax
		mov eax, dword ptr [edi+4]
		push eax
		mov ecx, ebx
		call get_register_value
		push eax
		call set_footer_coords
		jmp gameflow_continue;

		fail:
		cmp eax, 0x193ce200
		ja gameflow_above
		je gameflow_equal

		gameflow_above:
		jmp gameflow_above_offset

		gameflow_equal:
		jmp gameflow_equal_offset
	}
}

void gameflow_function_hook_install(void)
{
	char s[] = {0x68,0,0,0,0,0xc3};
	void * temp = gameflow_function_hook;
	memcpy (s+1, &temp, 4);
	memcpy ((void *)0x485015, s, 6);	
	
}

void fiction_printer_install(void)
{
	page_number = -1;

	char s[] = {0x68,0,0,0,0,0xc3};
	void * temp = fiction_printer_intercept;
	memcpy (s+1, &temp, 4);
	memcpy ((void *)0x493349, s, 6);	

	gameflow_function_hook_install();
}



// -----------------------------------------------


// rendering distance patches
void capship_distance_patch_install(void)
{
	*(float *)0x4d194c = 500000.0f;
}

void fighter_distance_patch_install(void)
{
	*(char *)0x425957 = 0xeb;
	*(char *)0x4259ab = 0xeb;
}

// menus
float sim_scale;

int navmap_render_top_x;
int navmap_render_top_y;
int navmap_render_bottom_x;
int navmap_render_bottom_y;
void * navmap_render_continue = (void *)0x472725;

void __declspec(naked) navmap_render_intercept(void)
{
	__asm
	{
		push navmap_render_bottom_y
		push navmap_render_bottom_x
		push navmap_render_top_y
		push navmap_render_top_x
		jmp navmap_render_continue
	}
}
void adjust_menus(void)
{
	int * res_x = (int*)0x43d3c6;
	int * res_y = (int*)0x43d3cf;

	if (*res_x > 640)
	{
		// ---------- Simulator ----------

			// "No Data Available"
			*(int *)0x49bf39 = *(int *)0x49bf39 + (*res_y - 480)/2;
			*(int *)0x49bf40 = *(int *)0x49bf40 + (*res_y - 480)/2;
			*(int *)0x49bf47 = *(int *)0x49bf47 + (*res_x - 640)/2;
			*(int *)0x49bf4e = *(int *)0x49bf4e + (*res_x - 640)/2;

			// "Your Score"
			*(int *)0x49bf86 = *(int *)0x49bf86 + (*res_x - 640)/2;
			*(int *)0x49bf8d = *(int *)0x49bf8d + (*res_y - 480);
			*(int *)0x49bf94 = *(int *)0x49bf94 + (*res_x - 640)/2;
			//*(int *)0x49bf9b = *(int *)0x49bf9b + (*res_y - 480)/2;  // not necessary??

			// Name and score
			*(int *)0x49be72 = *(int *)0x49be72 + (*res_y - 480)/2;
			*(int *)0x49be7e = *(int *)0x49be7e + (*res_x - 640)/2;
			*(int *)0x49bef0 = *(int *)0x49bef0 + (*res_y - 480)/2;
			*(int *)0x49bf03 = *(int *)0x49bf03 + (*res_x - 640)/2;

			// Ship rendering
			//*(int *)0x49b903 = *(int *)0x49b903 + (*res_x - 640)/2;  // not necessary??
			*(int *)0x49b8f0 = *(int *)0x49b8f0 + (*res_y - 480);
			*(int *)0x49b8fd = *(int *)0x49b8fd + (*res_x - 640);

			// Scale
			*(int *)0x49c118 = (int) (&sim_scale);
			if (*res_x == 1024)	sim_scale = 14.0f;
			if (*res_x == 800)	sim_scale = 10.0f;


		// ---------- Debrief ----------

			// Status
			*(int *)0x4e9c20 = *(int *)0x4e9c20 + (*res_x - 640)/2;
			*(int *)0x4e9c24 = *(int *)0x4e9c24 + (*res_y - 480)/2;

			// Mission Name
			*(int *)0x4e9bf0 = *(int *)0x4e9bf0 + (*res_x - 640)/2;
			*(int *)0x4e9bf4 = *(int *)0x4e9bf4 + (*res_y - 480)/2;

			// Overall Mission Status
			*(int *)0x4e9c08 = *(int *)0x4e9c08 + (*res_x - 640)/2;
			*(int *)0x4e9c0c = *(int *)0x4e9c0c + (*res_y - 480)/2;

			// Stats Names
			*(int *)0x4e9c00 = *(int *)0x4e9c00 + (*res_x - 640)/2;
			*(int *)0x4e9c04 = *(int *)0x4e9c04 + (*res_y - 480)/2;

			// Objective List (crashed before, so be careful)...
			*(int *)0x4e9bf8 = *(int *)0x4e9bf8 + (*res_x - 640)/2;
			*(int *)0x4e9bfc = *(int *)0x4e9bfc + (*res_y - 480)/2;

			// Stats Val (where's the X?)
			*(int *)0x4e9c1c = *(int *)0x4e9c1c + (*res_y - 480)/2;

			// SPENT HOURS ON THIS!!! Hope it's OK now
			// This centers everything else on this menu
			*(int *)0x4e9c10 = *(int *)0x4e9c10 + (*res_x - 480)/2 - 81;
			*(int *)0x4e9c18 = *(int *)0x4e9c18 + (*res_x - 480)/2 - 81;


		// ---------- Briefing ----------

			// Nav Descriptions
			*(int *)0x408e1d = *(int *)0x408e1d + (*res_y - 480)/2 + 30;
			*(int *)0x408e22 = *(int *)0x408e22 + (*res_x - 640)/2;


		// ---------- Navmap ----------

			// Rendered ship
			char t[] = {0x68,0,0,0,0,0xc3,0x8b,0x11};
			int address = (int)navmap_render_intercept;
			memcpy (t+1, &address, 4);
			memcpy ((void *) 0x47271f, t, 8);


			navmap_render_top_x = (*res_x - 640) / 2;
			navmap_render_top_y = (*res_y - 480) / 2;
			navmap_render_bottom_x = navmap_render_top_x + 239;
			navmap_render_bottom_y = navmap_render_top_y + 180;


		// ---------- Splash screens (UE mission completed screens) ----------

			char tt[] = {0x90,0x68,0,0,0,0,0x68,0,0,0,0,0x68,0x5e,0x3e,0x4a,0,0x68,0xd0,0x7a,0x40,0,0xc3};
			memcpy ((void *)0x4a3e49, tt, sizeof(tt));
			int tempx = (*res_x - 640) / 2;
			int tempy = (*res_y - 480) / 2;
			memcpy ((void *)0x4a3e4b, &tempy, 4);
			memcpy ((void *)0x4a3e50, &tempx, 4);

	}

}


// Standoff fonts patch 
typedef struct
{
	int offset;
	char * font;
	int color[3];
}	FONT;

FONT font[] = {
{0x40b06f, "ansi08aa", 0   , 0xff, 0   },
{0x4217d1, "euro20aa", 0xbe, 0x78, 0x96},
{0x4217fe, "euro20aa", 0xbe, 0xbe, 0x78},
{0x421c42, "ansi08aa", 0xbe, 0x78, 0x96},
{0x421c6f, "ansi08aa", 0xbe, 0xbe, 0x78},
{0x4226e0, "terminal", 0xc , 0x7b, 0xea},
{0x429237, "euro20aa", 0xff, 0xff, 0xff},
{0x43a9ca, "terminal", 0xff, 0xff, 0xff},
{0x43aa17, "terminal", 0xff, 0xff, 0xff},
{0x43c96f, "euro20aa", 0xf8, 0xf8, 0xf8},
/*{0x4408b6, "ansi08aa", 0x60, 0x60, 0xff},*/{0x4408b6, "ansi08aa", 0, 0xbb, 0},
{0x4414c8, "ansi08aa", 0x64, 0x64, 0x64},
{0x4451fa, "euro20aa", 0xff, 0xff, 0xff},
{0x4486e1, "mikity12", 0xbe, 0x78, 0x96},
{0x45925a, "terminal", 0xff, 0xff, 0xff},
/*{0x4626bd, "ansi08aa", 0x60, 0x60, 0xff},*/{0x4626bd, "ansi08aa", 0, 0xbb, 0},
{0x46534e, "euro20aa", 0xff, 0xff, 0xff},
{0x47656d, "ansi08aa", 0xff, 0xff, 0xff},
{0x476589, "ansi08aa", 0xff, 0   , 0   },
{0x4765a5, "ansi08aa", 0   , 0xff, 0   },
{0x4765c4, "ansi08aa", 0xff, 0xff, 0   },
{0x4765e6, "ansi08aa", 0x96, 0x96, 0x96},
{0x47ddef, "ansi08aa", 0   , 0xff, 0   },
{0x48244e, "ansi08aa", 0   , 0xff, 0   },
{0x483a1f, "terminal", 0   , 0   , 0xff},
{0x483a5d, "terminal", 0xff, 0   , 0   },
{0x483a7e, "terminal", 0x80, 0x80, 0x80},
{0x493368, "euro20aa", 0xf8, 0xf8, 0xf8},
{0x493a81, "euro20aa", 0xff, 0xff, 0xff},
{0x49ac04, "ansi08aa", 0x60, 0x60, 0xff},
{0x49ad84, "ansi08aa", 0x32, 0xc8, 0x64},
{0x49b6d2, "ansi08aa", 0xff, 0xff, 0xff},
/*{0x4a3653, "ansi08aa", 0x60, 0x60, 0xff},*/{0x4a3653, "ansi08aa", 0, 0xbb, 0},
{0x4a5309, "euro20aa", 0xff, 0xff, 0xff},
{0x4a6c7a, "euro20aa", 0xff, 0xff, 0xff},
/*{0x4aa3e0, "ansi08aa", 0x60, 0x60, 0xff},*/{0x4aa3e0, "ansi08aa", 0, 0xbb, 0},
/*{0x4afa71, "ansi08aa", 0x60, 0x60, 0xff},*/{0x4afa71, "ansi08aa", 0, 0xbb, 0},
{0x4b07fe, "euro20aa", 0xff, 0xff, 0xff},
{0x4b5f60, "euro20aa", 0xff, 0xff, 0xff},
{0x4b6b82, "euro20aa", 0xff, 0xff, 0xff},
{0x4b791f, "euro20aa", 0xff, 0xff, 0xff}
};

void * font_constructor_cont = (void *)0x44927a;

void font_patch_modify (int * font_info)
{
	int i;

	for (i = 0; i < sizeof(font) / sizeof(FONT); i++)
		if (font[i].offset == font_info[0])
		{
			font_info[1] = (int) font[i].font;
			font_info[2] = font[i].color[0];
			font_info[3] = font[i].color[1];
			font_info[4] = font[i].color[2];
		}
}

void __declspec(naked) font_intercept(void)
{
	__asm
	{
		//int 3
		pushad
		mov eax, esp
		add eax, 32
		push eax
		call font_patch_modify
		add esp, 4
		popad

		push ebp
		mov ebp, esp
		push esi
		mov esi, dword ptr[ebp+8]
		jmp font_constructor_cont
	}
}


// TRE patch 
void tre(void)
{
	void (*LoadTre)(char*,int) = (void(*)(char*, int)) 0x4166c0;
	int * selectedRes = (int *) 0x43d3c6;

	char func[20];
	char treName[20];
	int number;

	func[0] = 0;

	LoadTre("mission.tre", 2);
	LoadTre("data.tre", 1);
	LoadTre("speech.tre", 1);
	LoadTre("ifcomms.tre", 1);
	LoadTre("silence.tre", 2);
	LoadTre("music.tre", 1);


	FILE * f = fopen ("stan_1.cfg", "rt");
	if (f)
	{
		while (1)
		{
			fscanf (f, "%s %s %d\n", func, treName, &number);
			if (!strcmp(func, "end")) break;
			else if (!strcmp(func, "all") || (atoi(func) == *selectedRes))  
			{
				LoadTre (treName, number);
				add_tre_to_list(treName);
			}
		}
		fclose(f);
	}


	// NOW, INSTALL PATCHES THAT CAN ONLY BE INSTALLED WHEN INFO REGARDING TRES HAS BEEN GATHERED

	// load vdu patch here
	vdu_install();

	// gun convergence
	install_gun_convergence_patch();
}

bool menu = false;

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{

HMODULE lib = NULL;
void (*doMenu)(void);
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		
		if (!menu)
		{

			{
				// - TRE loader
				void (*temp)(void) = tre;
				char treBridge[] = {0x90, 0x68, 0xa8, 0xac,0x43,0, 0x68, 0,0,0,0,0xc3};
				memcpy((void *)0x43ac4e, treBridge, 12);
				memcpy((void *)0x43ac55, &temp, 4);
			}

			{
				// - command interpreter hook, for keyboard functions (and more)
				void (*temp)(void) = (void(*)(void)) ((unsigned int)kb_mci_hook - (unsigned int)0x4854eb);
				char kb_mci_bridge[] = {0xe9,0,0,0,0};
				memcpy (kb_mci_bridge+1, &temp, 4);
				memcpy ((void*)0x4854e6, kb_mci_bridge, 5);
			}

			{
				// - hook for intercepting keystrokes in spaceflight (part of keyboard patch)
				void (*temp)(void) = (void (*) (void)) keyboard_hook;
				char kb_hook_bridge[] = {0x68,0,0,0,0,0xc3};
				memcpy (kb_hook_bridge+1, &temp, 4);
				memcpy ((void *)0x446378, kb_hook_bridge, 6);
			}

			fiction_printer_install();

			// -----------------------------------------------------------------------------

			// -----------------------------------------------------------------------------

			
			{

				// fonts
				void (*temp)(void) = font_intercept;
				char fontBridge[] = {0x90,0x68, 0,0,0,0,0xc3};
				memcpy((void *)0x449250, fontBridge, 12);
				memcpy((void *)0x449252, &temp, 4);

					// damage menu
					*(int *)0x422f18 = 0;		// blue
					*(int *)0x422f1d = 0;		// red
					*(int *)0x422f44 = 0xbb;	// green

					// nav
					*(char *) 0x46f9ea = 0x78;	// address (see 43d9b9)


			}

			install_divx_patch();

			{
				// Multiple UV2 hook
				void (*temp)(void) = (void(*)(void)) uv2_hook;
				char uv2_hook_bridge[] = {0x68,0,0,0,0,0xc3};
				memcpy (uv2_hook_bridge+1, &temp, 4);
				memcpy ((void*)0x4431da, uv2_hook_bridge, 6);

			}

			{
				// UV2 hook part2
				void (*temp)(void) = (void(*)(void)) uv2_hook2;
				char uv2_hook_bridge[] = {0x68,0,0,0,0,0xc3};
				memcpy (uv2_hook_bridge+1, &temp, 4);
				memcpy ((void*)0x442b76, uv2_hook_bridge, 6);

			}

/*			{
				// UV2 hook part3 (EXPERIMENTAL!!)
				void (*temp)(void) = (void(*)(void)) uv2_hook3;
				char uv2_hook_bridge[] = {0x68,0,0,0,0,0xc3};
				memcpy (uv2_hook_bridge+1, &temp, 4);
				memcpy ((void*)0x477D13, uv2_hook_bridge, 6);

			}
*/


			{

				// eliminate black bars on credits screen
				void (*temp)(void) = 0;
				memcpy ((void *) 0x421b03, &temp, 4);
				memcpy ((void *) 0x421b08, &temp, 4);
				memcpy ((void *) 0x421afc, &temp, 4);
				memcpy ((void *) 0x421ad3, &temp, 4);
				memcpy ((void *) 0x421acf, &temp, 1);
			}


			// rotation patch
			char rotation[] = {0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90};
			memcpy((void *)0x475733, rotation, 7);


			// increasing number of asteroids
			// rocks  434849
			*(int *)0x434849 = 20000;

			// 64mb heap, just in case
			//*(int *)0x4563d0 = *(int *)0x4563d5 = 64*1024*1024;
			*(int *)0x4563d0 = 0;
			*(int *)0x4563d5 = 64*1024*1024;

			

			// show menu
			lib = LoadLibrary("standoff_menu.dll");
			doMenu = (void (*)()) GetProcAddress(lib,"doMenu");
			if (doMenu)
				doMenu();
			else
				MessageBox(0,"Error loading menu!","ERROR",MB_ICONERROR);
			FreeLibrary(lib);

			fighter_distance_patch_install();
			capship_distance_patch_install();
			adjust_menus();
			menu = true;
		}

		break;
		
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
		break;
    }
    return TRUE;
}


// This is an example of an exported variable
WCSOHR_API int nWcsohr=0;

// This is an example of an exported function.
WCSOHR_API int fnWcsohr(void)
{
	return 42;
}

// This is the constructor of a class that has been exported.
// see wcsohr.h for the class definition
CWcsohr::CWcsohr()
{ 
	return; 
}

